@@ -3,7 +3,7 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-// spell-checker:ignore (ToDO) sourcepath targetpath nushell canonicalized
+// spell-checker:ignore (ToDO) sourcepath targetpath nushell canonicalized unwriteable
 
 mod error;
 #[cfg(unix)]
@@ -20,11 +20,11 @@ use std::collections::HashSet;
 use std::env;
 use std::ffi::OsString;
 use std::fs;
-use std::io;
+use std::io::{self, IsTerminal};
 #[cfg(unix)]
 use std::os::unix;
 #[cfg(unix)]
-use std::os::unix::fs::FileTypeExt;
+use std::os::unix::fs::{FileTypeExt, PermissionsExt};
 #[cfg(windows)]
 use std::os::windows;
 use std::path::{Path, PathBuf, absolute};
@@ -38,6 +38,8 @@ use uucore::backup_control::{self, source_is_target_backup};
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult, USimpleError, UUsageError, set_exit_code};
 #[cfg(unix)]
+use uucore::fs::display_permissions_unix;
+#[cfg(unix)]
 use uucore::fs::make_fifo;
 use uucore::fs::{
     MissingHandling, ResolveMode, are_hardlinks_or_one_way_symlink_to_same_file,
@@ -128,12 +130,14 @@ impl Default for Options {
 /// specifies behavior of the overwrite flag
 #[derive(Clone, Debug, Eq, PartialEq, Default)]
 pub enum OverwriteMode {
+    /// No flag specified - prompt for unwriteable files when stdin is TTY
+    #[default]
+    Default,
     /// '-n' '--no-clobber'   do not overwrite
     NoClobber,
     /// '-i' '--interactive'  prompt before overwrite
     Interactive,
     ///'-f' '--force'         overwrite without prompt
-    #[default]
     Force,
 }
 
@@ -341,8 +345,10 @@ fn determine_overwrite_mode(matches: &ArgMatches) -> OverwriteMode {
         OverwriteMode::NoClobber
     } else if matches.get_flag(OPT_INTERACTIVE) {
         OverwriteMode::Interactive
-    } else {
+    } else if matches.get_flag(OPT_FORCE) {
         OverwriteMode::Force
+    } else {
+        OverwriteMode::Default
     }
 }
 
@@ -421,15 +427,14 @@ fn handle_two_paths(source: &Path, target: &Path, opts: &Options) -> UResult<()>
     } else if target.exists() && source_is_dir {
         match opts.overwrite {
             OverwriteMode::NoClobber => return Ok(()),
-            OverwriteMode::Interactive => {
-                if !prompt_yes!(
-                    "{}",
-                    translate!("mv-prompt-overwrite", "target" => target.quote())
-                ) {
-                    return Err(io::Error::other("").into());
+            OverwriteMode::Interactive => prompt_overwrite(target, None)?,
+            OverwriteMode::Force => {}
+            OverwriteMode::Default => {
+                let (writable, mode) = is_writable(target);
+                if !writable && std::io::stdin().is_terminal() {
+                    prompt_overwrite(target, mode)?;
                 }
             }
-            OverwriteMode::Force => {}
         }
         Err(MvError::NonDirectoryToDirectory(
             source.quote().to_string(),
@@ -648,7 +653,7 @@ fn move_files_into_dir(files: &[PathBuf], target_dir: &Path, options: &Options)
             // If the target file was already created in this mv call, do not overwrite
             show!(USimpleError::new(
                 1,
-                translate!("mv-error-will-not-overwrite-just-created", "target" => targetpath.display(), "source" => sourcepath.display()),
+                translate!("mv-error-will-not-overwrite-just-created", "target" => targetpath.quote(), "source" => sourcepath.quote()),
             ));
             continue;
         }
@@ -731,15 +736,15 @@ fn rename(
                 }
                 return Ok(());
             }
-            OverwriteMode::Interactive => {
-                if !prompt_yes!(
-                    "{}",
-                    translate!("mv-prompt-overwrite", "target" => to.quote())
-                ) {
-                    return Err(io::Error::other(""));
+            OverwriteMode::Interactive => prompt_overwrite(to, None)?,
+            OverwriteMode::Force => {}
+            OverwriteMode::Default => {
+                // GNU mv prompts when stdin is a TTY and target is not writable
+                let (writable, mode) = is_writable(to);
+                if !writable && std::io::stdin().is_terminal() {
+                    prompt_overwrite(to, mode)?;
                 }
             }
-            OverwriteMode::Force => {}
         }
 
         backup_path = backup_control::get_backup_path(opts.backup, to, &opts.suffix);
@@ -903,7 +908,12 @@ fn rename_fifo_fallback(_from: &Path, _to: &Path) -> io::Result<()> {
 #[cfg(unix)]
 fn rename_symlink_fallback(from: &Path, to: &Path) -> io::Result<()> {
     let path_symlink_points_to = fs::read_link(from)?;
-    unix::fs::symlink(path_symlink_points_to, to).and_then(|_| fs::remove_file(from))
+    unix::fs::symlink(path_symlink_points_to, to)?;
+    #[cfg(not(any(target_os = "macos", target_os = "redox")))]
+    {
+        let _ = copy_xattrs_if_supported(from, to);
+    }
+    fs::remove_file(from)
 }
 
 #[cfg(windows)]
@@ -1022,7 +1032,7 @@ fn copy_dir_contents(
     }
     #[cfg(not(unix))]
     {
-        copy_dir_contents_recursive(from, to, None, None, verbose, progress_bar, display_manager)?;
+        copy_dir_contents_recursive(from, to, verbose, progress_bar, display_manager)?;
     }
 
     Ok(())
@@ -1033,8 +1043,6 @@ fn copy_dir_contents_recursive(
     to_dir: &Path,
     #[cfg(unix)] hardlink_tracker: &mut HardlinkTracker,
     #[cfg(unix)] hardlink_scanner: &HardlinkGroupScanner,
-    #[cfg(not(unix))] _hardlink_tracker: Option<()>,
-    #[cfg(not(unix))] _hardlink_scanner: Option<()>,
     verbose: bool,
     progress_bar: Option<&ProgressBar>,
     display_manager: Option<&MultiProgress>,
@@ -1073,10 +1081,6 @@ fn copy_dir_contents_recursive(
                 hardlink_tracker,
                 #[cfg(unix)]
                 hardlink_scanner,
-                #[cfg(not(unix))]
-                _hardlink_tracker,
-                #[cfg(not(unix))]
-                _hardlink_scanner,
                 verbose,
                 progress_bar,
                 display_manager,
@@ -1094,7 +1098,13 @@ fn copy_dir_contents_recursive(
             }
             #[cfg(not(unix))]
             {
-                fs::copy(&from_path, &to_path)?;
+                if from_path.is_symlink() {
+                    // Copy a symlink file (no-follow).
+                    rename_symlink_fallback(&from_path, &to_path)?;
+                } else {
+                    // Copy a regular file.
+                    fs::copy(&from_path, &to_path)?;
+                }
             }
 
             // Print verbose message for file
@@ -1137,14 +1147,17 @@ fn copy_file_with_hardlinks_helper(
         return Ok(());
     }
 
-    // Regular file copy
-    #[cfg(all(unix, not(any(target_os = "macos", target_os = "redox"))))]
-    {
-        fs::copy(from, to).and_then(|_| fsxattr::copy_xattrs(&from, &to))?;
-    }
-    #[cfg(any(target_os = "macos", target_os = "redox"))]
-    {
+    if from.is_symlink() {
+        // Copy a symlink file (no-follow).
+        rename_symlink_fallback(from, to)?;
+    } else {
+        // Copy a regular file.
         fs::copy(from, to)?;
+        // Copy xattrs, ignoring ENOTSUP errors (filesystem doesn't support xattrs)
+        #[cfg(all(unix, not(any(target_os = "macos", target_os = "redox"))))]
+        {
+            let _ = copy_xattrs_if_supported(from, to);
+        }
     }
 
     Ok(())
@@ -1159,7 +1172,7 @@ fn rename_file_fallback(
     // Remove existing target file if it exists
     if to.is_symlink() {
         fs::remove_file(to).map_err(|err| {
-            let inter_device_msg = translate!("mv-error-inter-device-move-failed", "from" => from.display(), "to" => to.display(), "err" => err);
+            let inter_device_msg = translate!("mv-error-inter-device-move-failed", "from" => from.quote(), "to" => to.quote(), "err" => err);
             io::Error::new(err.kind(), inter_device_msg)
         })?;
     } else if to.exists() {
@@ -1185,22 +1198,88 @@ fn rename_file_fallback(
     }
 
     // Regular file copy
-    #[cfg(all(unix, not(any(target_os = "macos", target_os = "redox"))))]
     fs::copy(from, to)
-        .and_then(|_| fsxattr::copy_xattrs(&from, &to))
-        .and_then(|_| fs::remove_file(from))
         .map_err(|err| io::Error::new(err.kind(), translate!("mv-error-permission-denied")))?;
-    #[cfg(any(target_os = "macos", target_os = "redox", not(unix)))]
-    fs::copy(from, to)
-        .and_then(|_| fs::remove_file(from))
+
+    // Copy xattrs, ignoring ENOTSUP errors (filesystem doesn't support xattrs)
+    #[cfg(all(unix, not(any(target_os = "macos", target_os = "redox"))))]
+    {
+        let _ = copy_xattrs_if_supported(from, to);
+    }
+
+    fs::remove_file(from)
         .map_err(|err| io::Error::new(err.kind(), translate!("mv-error-permission-denied")))?;
     Ok(())
 }
 
+/// Copy xattrs from source to destination, ignoring ENOTSUP/EOPNOTSUPP errors.
+/// These errors indicate the filesystem doesn't support extended attributes,
+/// which is acceptable when moving files across filesystems.
+#[cfg(all(unix, not(any(target_os = "macos", target_os = "redox"))))]
+fn copy_xattrs_if_supported(from: &Path, to: &Path) -> io::Result<()> {
+    match fsxattr::copy_xattrs(from, to) {
+        Ok(()) => Ok(()),
+        Err(e) if e.raw_os_error() == Some(libc::EOPNOTSUPP) => Ok(()),
+        Err(e) => Err(e),
+    }
+}
+
 fn is_empty_dir(path: &Path) -> bool {
     fs::read_dir(path).is_ok_and(|mut contents| contents.next().is_none())
 }
 
+/// Check if file is writable, returning the mode for potential reuse.
+#[cfg(unix)]
+fn is_writable(path: &Path) -> (bool, Option<u32>) {
+    if let Ok(metadata) = path.metadata() {
+        let mode = metadata.permissions().mode();
+        // Check if user write bit is set
+        ((mode & 0o200) != 0, Some(mode))
+    } else {
+        (false, None) // If we can't get metadata, prompt user to be safe
+    }
+}
+
+/// Check if file is writable.
+#[cfg(not(unix))]
+fn is_writable(path: &Path) -> (bool, Option<u32>) {
+    if let Ok(metadata) = path.metadata() {
+        (!metadata.permissions().readonly(), None)
+    } else {
+        (false, None) // If we can't get metadata, prompt user to be safe
+    }
+}
+
+#[cfg(unix)]
+fn get_interactive_prompt(to: &Path, cached_mode: Option<u32>) -> String {
+    use libc::mode_t;
+    // Use cached mode if available, otherwise fetch it
+    let mode = cached_mode.or_else(|| to.metadata().ok().map(|m| m.permissions().mode()));
+    if let Some(mode) = mode {
+        let file_mode = mode & 0o777;
+        // Check if file is not writable by user
+        if (mode & 0o200) == 0 {
+            let perms = display_permissions_unix(mode as mode_t, false);
+            let mode_info = format!("{file_mode:04o} ({perms})");
+            return translate!("mv-prompt-overwrite-mode", "target" => to.quote(), "mode_info" => mode_info);
+        }
+    }
+    translate!("mv-prompt-overwrite", "target" => to.quote())
+}
+
+#[cfg(not(unix))]
+fn get_interactive_prompt(to: &Path, _cached_mode: Option<u32>) -> String {
+    translate!("mv-prompt-overwrite", "target" => to.quote())
+}
+
+/// Prompts the user for confirmation and returns an error if declined.
+fn prompt_overwrite(to: &Path, cached_mode: Option<u32>) -> io::Result<()> {
+    if !prompt_yes!("{}", get_interactive_prompt(to, cached_mode)) {
+        return Err(io::Error::other(""));
+    }
+    Ok(())
+}
+
 /// Checks if a file can be deleted by attempting to open it with delete permissions.
 #[cfg(windows)]
 fn can_delete_file(path: &Path) -> bool {
