@@ -26,14 +26,16 @@ use args::{FilterMode, Settings, Signum, parse_args};
 use chunks::ReverseChunks;
 use follow::Observer;
 use memchr::{memchr_iter, memrchr_iter};
-use paths::{FileExtTail, HeaderPrinter, Input, InputKind, MetadataExtTail};
+use paths::{FileExtTail, HeaderPrinter, Input, InputKind};
 use same_file::Handle;
 use std::cmp::Ordering;
 use std::fs::File;
 use std::io::{self, BufReader, BufWriter, ErrorKind, Read, Seek, SeekFrom, Write, stdin, stdout};
 use std::path::{Path, PathBuf};
 use uucore::display::Quotable;
-use uucore::error::{FromIo, UResult, USimpleError, get_exit_code, set_exit_code};
+use uucore::error::{FromIo, UResult, USimpleError, set_exit_code};
+use uucore::translate;
+
 use uucore::{show, show_error};
 
 #[uucore::main]
@@ -46,7 +48,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         args::VerificationResult::CannotFollowStdinByName => {
             return Err(USimpleError::new(
                 1,
-                format!("cannot follow {} by name", text::DASH.quote()),
+                translate!("tail-error-cannot-follow-stdin-by-name", "stdin" => text::DASH.quote()),
             ));
         }
         // Exit early if we do not output anything. Note, that this may break a pipe
@@ -63,6 +65,16 @@ fn uu_tail(settings: &Settings) -> UResult<()> {
     let mut observer = Observer::from(settings);
 
     observer.start(settings)?;
+
+    // Print debug info about the follow implementation being used
+    if settings.debug && settings.follow.is_some() {
+        if observer.use_polling {
+            show_error!("{}", translate!("tail-debug-using-polling-mode"));
+        } else {
+            show_error!("{}", translate!("tail-debug-using-notification-mode"));
+        }
+    }
+
     // Do an initial tail print of each path's content.
     // Add `path` and `reader` to `files` map if `--follow` is selected.
     for input in &settings.inputs.clone() {
@@ -94,10 +106,6 @@ fn uu_tail(settings: &Settings) -> UResult<()> {
         }
     }
 
-    if get_exit_code() > 0 && paths::stdin_is_bad_fd() {
-        show_error!("-: {}", text::BAD_FD);
-    }
-
     Ok(())
 }
 
@@ -112,27 +120,29 @@ fn tail_file(
     if !path.exists() {
         set_exit_code(1);
         show_error!(
-            "cannot open '{}' for reading: {}",
-            input.display_name,
-            text::NO_SUCH_FILE
+            "{}",
+            translate!("tail-error-cannot-open-no-such-file", "file" => input.display_name.clone(), "error" => translate!("tail-no-such-file-or-directory"))
         );
         observer.add_bad_path(path, input.display_name.as_str(), false)?;
     } else if path.is_dir() {
         set_exit_code(1);
 
         header_printer.print_input(input);
-        let err_msg = "Is a directory".to_string();
+        let err_msg = translate!("tail-is-a-directory");
 
-        show_error!("error reading '{}': {err_msg}", input.display_name);
+        show_error!(
+            "{}",
+            translate!("tail-error-reading-file", "file" => input.display_name.clone(), "error" => err_msg)
+        );
         if settings.follow.is_some() {
             let msg = if settings.retry {
                 ""
             } else {
-                "; giving up on this name"
+                &translate!("tail-giving-up-on-this-name")
             };
             show_error!(
-                "{}: cannot follow end of this type of file{msg}",
-                input.display_name,
+                "{}",
+                translate!("tail-error-cannot-follow-file-type", "file" => input.display_name.clone(), "msg" => msg)
             );
         }
         if !observer.follow_name_retry() {
@@ -140,15 +150,21 @@ fn tail_file(
             return Ok(());
         }
         observer.add_bad_path(path, input.display_name.as_str(), false)?;
-    } else if input.is_tailable() {
-        let metadata = path.metadata().ok();
-        match File::open(path) {
+    } else {
+        #[cfg(unix)]
+        let open_result = open_file(path, settings.pid != 0);
+        #[cfg(not(unix))]
+        let open_result = File::open(path);
+
+        match open_result {
             Ok(mut file) => {
+                let st = file.metadata()?;
+                let blksize_limit = uucore::fs::sane_blksize::sane_blksize_from_metadata(&st);
                 header_printer.print_input(input);
                 let mut reader;
                 if !settings.presume_input_pipe
                     && file.is_seekable(if input.is_stdin() { offset } else { 0 })
-                    && metadata.as_ref().unwrap().get_block_size() > 0
+                    && (!st.is_file() || st.len() > blksize_limit)
                 {
                     bounded_tail(&mut file, settings);
                     reader = BufReader::new(file);
@@ -156,33 +172,72 @@ fn tail_file(
                     reader = BufReader::new(file);
                     unbounded_tail(&mut reader, settings)?;
                 }
-                observer.add_path(
-                    path,
-                    input.display_name.as_str(),
-                    Some(Box::new(reader)),
-                    true,
-                )?;
+                if input.is_tailable() {
+                    observer.add_path(
+                        path,
+                        input.display_name.as_str(),
+                        Some(Box::new(reader)),
+                        true,
+                    )?;
+                } else {
+                    observer.add_bad_path(path, input.display_name.as_str(), false)?;
+                }
             }
             Err(e) if e.kind() == ErrorKind::PermissionDenied => {
                 observer.add_bad_path(path, input.display_name.as_str(), false)?;
                 show!(e.map_err_context(|| {
-                    format!("cannot open '{}' for reading", input.display_name)
+                    translate!("tail-error-cannot-open-for-reading", "file" => input.display_name.clone())
                 }));
             }
             Err(e) => {
                 observer.add_bad_path(path, input.display_name.as_str(), false)?;
                 return Err(e.map_err_context(|| {
-                    format!("cannot open '{}' for reading", input.display_name)
+                    translate!("tail-error-cannot-open-for-reading", "file" => input.display_name.clone())
                 }));
             }
         }
-    } else {
-        observer.add_bad_path(path, input.display_name.as_str(), false)?;
     }
 
     Ok(())
 }
 
+/// Opens a file, using non-blocking mode for FIFOs when `use_nonblock_for_fifo` is true.
+///
+/// When opening a FIFO with `--pid`, we need to use O_NONBLOCK so that:
+/// 1. The open() call doesn't block waiting for a writer
+/// 2. We can periodically check if the monitored process is still alive
+///
+/// After opening, we clear O_NONBLOCK so subsequent reads block normally.
+/// Without `--pid`, FIFOs block on open() until a writer connects (GNU behavior).
+#[cfg(unix)]
+fn open_file(path: &Path, use_nonblock_for_fifo: bool) -> std::io::Result<File> {
+    use nix::fcntl::{FcntlArg, OFlag, fcntl};
+    use std::fs::OpenOptions;
+    use std::os::fd::AsFd;
+    use std::os::unix::fs::{FileTypeExt, OpenOptionsExt};
+
+    let is_fifo = path
+        .metadata()
+        .ok()
+        .is_some_and(|m| m.file_type().is_fifo());
+
+    if is_fifo && use_nonblock_for_fifo {
+        let file = OpenOptions::new()
+            .read(true)
+            .custom_flags(libc::O_NONBLOCK)
+            .open(path)?;
+
+        // Clear O_NONBLOCK so reads block normally
+        let flags = fcntl(file.as_fd(), FcntlArg::F_GETFL)?;
+        let new_flags = OFlag::from_bits_truncate(flags) & !OFlag::O_NONBLOCK;
+        fcntl(file.as_fd(), FcntlArg::F_SETFL(new_flags))?;
+
+        Ok(file)
+    } else {
+        File::open(path)
+    }
+}
+
 fn tail_stdin(
     settings: &Settings,
     header_printer: &mut HeaderPrinter,
@@ -202,9 +257,8 @@ fn tail_stdin(
                 if meta.file_type().is_dir() {
                     set_exit_code(1);
                     show_error!(
-                        "cannot open '{}' for reading: {}",
-                        input.display_name,
-                        text::NO_SUCH_FILE
+                        "{}",
+                        translate!("tail-error-cannot-open-no-such-file", "file" => input.display_name.clone(), "error" => translate!("tail-no-such-file-or-directory"))
                     );
                     return Ok(());
                 }
@@ -212,6 +266,17 @@ fn tail_stdin(
         }
     }
 
+    // Check if stdin was closed before Rust reopened it as /dev/null
+    if paths::stdin_is_bad_fd() {
+        set_exit_code(1);
+        show_error!(
+            "{}",
+            translate!("tail-error-cannot-fstat", "file" => translate!("tail-stdin-header").quote(), "error" => translate!("tail-bad-fd"))
+        );
+        show_error!("{}", translate!("tail-no-files-remaining"));
+        return Ok(());
+    }
+
     match input.resolve() {
         // fifo
         Some(path) => {
@@ -240,24 +305,21 @@ fn tail_stdin(
             if paths::stdin_is_bad_fd() {
                 set_exit_code(1);
                 show_error!(
-                    "cannot fstat {}: {}",
-                    text::STDIN_HEADER.quote(),
-                    text::BAD_FD
+                    "{}",
+                    translate!("tail-error-cannot-fstat", "file" => translate!("tail-stdin-header"), "error" => translate!("tail-bad-fd"))
                 );
                 if settings.follow.is_some() {
                     show_error!(
-                        "error reading {}: {}",
-                        text::STDIN_HEADER.quote(),
-                        text::BAD_FD
+                        "{}",
+                        translate!("tail-error-reading-file", "file" => translate!("tail-stdin-header"), "error" => translate!("tail-bad-fd"))
                     );
                 }
             } else {
                 let mut reader = BufReader::new(stdin());
                 unbounded_tail(&mut reader, settings)?;
-                observer.add_stdin(input.display_name.as_str(), Some(Box::new(reader)), true)?;
             }
         }
-    };
+    }
 
     Ok(())
 }
@@ -341,7 +403,7 @@ fn forwards_thru_file(
                 }
                 total += n;
             }
-            Err(e) if e.kind() == ErrorKind::Interrupted => continue,
+            Err(e) if e.kind() == ErrorKind::Interrupted => (),
             Err(e) => return Err(e),
         }
     }
@@ -351,6 +413,10 @@ fn forwards_thru_file(
 /// `num_delimiters` instance of `delimiter`. The `file` is left seek'd to the
 /// position just after that delimiter.
 fn backwards_thru_file(file: &mut File, num_delimiters: u64, delimiter: u8) {
+    if num_delimiters == 0 {
+        file.seek(SeekFrom::End(0)).unwrap();
+        return;
+    }
     // This variable counts the number of delimiters found in the file
     // so far (reading from the end of the file toward the beginning).
     let mut counter = 0;
@@ -396,6 +462,7 @@ fn backwards_thru_file(file: &mut File, num_delimiters: u64, delimiter: u8) {
 /// being a nice performance win for very large files.
 fn bounded_tail(file: &mut File, settings: &Settings) {
     debug_assert!(!settings.presume_input_pipe);
+    let mut limit = None;
 
     // Find the position in the file to start printing from.
     match &settings.mode {
@@ -407,12 +474,13 @@ fn bounded_tail(file: &mut File, settings: &Settings) {
             file.seek(SeekFrom::Start(i as u64)).unwrap();
         }
         FilterMode::Lines(Signum::MinusZero, _) => {
-            return;
+            file.seek(SeekFrom::End(0)).unwrap();
         }
         FilterMode::Bytes(Signum::Negative(count)) => {
-            let len = file.seek(SeekFrom::End(0)).unwrap();
-            file.seek(SeekFrom::End(-((*count).min(len) as i64)))
-                .unwrap();
+            if file.seek(SeekFrom::End(-(*count as i64))).is_err() {
+                file.seek(SeekFrom::Start(0)).unwrap();
+            }
+            limit = Some(*count);
         }
         FilterMode::Bytes(Signum::Positive(count)) if count > &1 => {
             // GNU `tail` seems to index bytes and lines starting at 1, not
@@ -420,15 +488,12 @@ fn bounded_tail(file: &mut File, settings: &Settings) {
             file.seek(SeekFrom::Start(*count - 1)).unwrap();
         }
         FilterMode::Bytes(Signum::MinusZero) => {
-            return;
+            file.seek(SeekFrom::End(0)).unwrap();
         }
         _ => {}
     }
 
-    // Print the target section of the file.
-    let stdout = stdout();
-    let mut stdout = stdout.lock();
-    io::copy(file, &mut stdout).unwrap();
+    print_target_section(file, limit);
 }
 
 fn unbounded_tail<T: Read>(reader: &mut BufReader<T>, settings: &Settings) -> UResult<()> {
@@ -437,7 +502,7 @@ fn unbounded_tail<T: Read>(reader: &mut BufReader<T>, settings: &Settings) -> UR
         FilterMode::Lines(Signum::Negative(count), sep) => {
             let mut chunks = chunks::LinesChunkBuffer::new(*sep, *count);
             chunks.fill(reader)?;
-            chunks.print(&mut writer)?;
+            chunks.write(&mut writer)?;
         }
         FilterMode::Lines(Signum::PlusZero | Signum::Positive(1), _) => {
             io::copy(reader, &mut writer)?;
@@ -454,7 +519,7 @@ fn unbounded_tail<T: Read>(reader: &mut BufReader<T>, settings: &Settings) -> UR
                 }
             }
             if chunk.has_data() {
-                chunk.print_lines(&mut writer, num_skip as usize)?;
+                chunk.write_lines(&mut writer, num_skip as usize)?;
                 io::copy(reader, &mut writer)?;
             }
         }
@@ -463,6 +528,11 @@ fn unbounded_tail<T: Read>(reader: &mut BufReader<T>, settings: &Settings) -> UR
             chunks.fill(reader)?;
             chunks.print(&mut writer)?;
         }
+        FilterMode::Lines(Signum::MinusZero, sep) => {
+            let mut chunks = chunks::LinesChunkBuffer::new(*sep, 0);
+            chunks.fill(reader)?;
+            chunks.write(&mut writer)?;
+        }
         FilterMode::Bytes(Signum::PlusZero | Signum::Positive(1)) => {
             io::copy(reader, &mut writer)?;
         }
@@ -491,10 +561,34 @@ fn unbounded_tail<T: Read>(reader: &mut BufReader<T>, settings: &Settings) -> UR
         }
         _ => {}
     }
+    #[cfg(not(target_os = "windows"))]
     writer.flush()?;
+
+    // SIGPIPE is not available on Windows.
+    #[cfg(target_os = "windows")]
+    writer.flush().inspect_err(|err| {
+        if err.kind() == ErrorKind::BrokenPipe {
+            std::process::exit(13);
+        }
+    })?;
     Ok(())
 }
 
+fn print_target_section<R>(file: &mut R, limit: Option<u64>)
+where
+    R: Read + ?Sized,
+{
+    // Print the target section of the file.
+    let stdout = stdout();
+    let mut stdout = stdout.lock();
+    if let Some(limit) = limit {
+        let mut reader = file.take(limit);
+        io::copy(&mut reader, &mut stdout).unwrap();
+    } else {
+        io::copy(file, &mut stdout).unwrap();
+    }
+}
+
 #[cfg(test)]
 mod tests {
 
