@@ -5,17 +5,16 @@
 
 // spell-checker:ignore (paths) atim sublink subwords azerty azeaze xcwww azeaz amaz azea qzerty tazerty tsublink testfile1 testfile2 filelist fpath testdir testfile
 // spell-checker:ignore selfref ELOOP smallfile
+
 #[cfg(not(windows))]
 use regex::Regex;
 
-use uutests::at_and_ucmd;
-use uutests::new_ucmd;
 #[cfg(not(target_os = "windows"))]
 use uutests::unwrap_or_return;
 use uutests::util::TestScenario;
 #[cfg(not(target_os = "windows"))]
 use uutests::util::expected_result;
-use uutests::util_name;
+use uutests::{at_and_ucmd, new_ucmd, util_name};
 
 #[cfg(not(target_os = "openbsd"))]
 const SUB_DIR: &str = "subdir/deeper";
@@ -283,6 +282,120 @@ fn test_du_env_block_size_hierarchy() {
     assert_eq!(expected, result2);
 }
 
+#[test]
+fn test_du_binary_block_size() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+    let dir = "a";
+
+    at.mkdir(dir);
+    let fpath = at.plus(format!("{dir}/file"));
+    std::fs::File::create(&fpath)
+        .expect("cannot create test file")
+        .set_len(100_000)
+        .expect("cannot set file size");
+
+    let test_cases = [
+        ("0b1", "1"),
+        ("0b10100", "20"),
+        ("0b1000000000", "512"),
+        ("0b10K", "2K"),
+    ];
+
+    for (binary, decimal) in test_cases {
+        let decimal = ts
+            .ucmd()
+            .arg(dir)
+            .arg(format!("--block-size={decimal}"))
+            .succeeds()
+            .stdout_move_str();
+
+        let binary = ts
+            .ucmd()
+            .arg(dir)
+            .arg(format!("--block-size={binary}"))
+            .succeeds()
+            .stdout_move_str();
+
+        assert_eq!(
+            decimal, binary,
+            "Binary {binary} should equal decimal {decimal}"
+        );
+    }
+}
+
+#[test]
+fn test_du_binary_env_block_size() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+    let dir = "a";
+
+    at.mkdir(dir);
+    let fpath = at.plus(format!("{dir}/file"));
+    std::fs::File::create(&fpath)
+        .expect("cannot create test file")
+        .set_len(100_000)
+        .expect("cannot set file size");
+
+    let expected = ts
+        .ucmd()
+        .arg(dir)
+        .arg("--block-size=1024")
+        .succeeds()
+        .stdout_move_str();
+
+    let result = ts
+        .ucmd()
+        .arg(dir)
+        .env("DU_BLOCK_SIZE", "0b10000000000")
+        .succeeds()
+        .stdout_move_str();
+
+    assert_eq!(expected, result);
+}
+
+#[test]
+fn test_du_invalid_binary_size() {
+    let ts = TestScenario::new(util_name!());
+
+    ts.ucmd()
+        .arg("--block-size=0b123")
+        .arg("/tmp")
+        .fails_with_code(1)
+        .stderr_only("du: invalid suffix in --block-size argument '0b123'\n");
+
+    ts.ucmd()
+        .arg("--threshold=0b123")
+        .arg("/tmp")
+        .fails_with_code(1)
+        .stderr_only("du: invalid suffix in --threshold argument '0b123'\n");
+}
+
+#[test]
+fn test_du_binary_edge_cases() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+    at.write("foo", "test");
+
+    ts.ucmd()
+        .arg("-B0b")
+        .arg("foo")
+        .fails()
+        .stderr_only("du: invalid --block-size argument '0b'\n");
+
+    ts.ucmd()
+        .arg("-B0B")
+        .arg("foo")
+        .fails()
+        .stderr_only("du: invalid suffix in --block-size argument '0B'\n");
+
+    ts.ucmd()
+        .arg("--block-size=0b1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111")
+        .arg("foo")
+        .fails_with_code(1)
+        .stderr_contains("too large");
+}
+
 #[test]
 fn test_du_non_existing_files() {
     new_ucmd!()
@@ -691,6 +804,44 @@ fn test_du_inodes_with_count_links_all() {
     assert_eq!(result_seq, ["1\td/d", "1\td/f", "1\td/h", "4\td"]);
 }
 
+#[cfg(not(target_os = "android"))]
+#[test]
+fn test_du_count_links_hardlinks_separately() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+
+    at.mkdir("dir");
+    at.touch("dir/file");
+    at.hard_link("dir/file", "dir/hard_link");
+
+    let result_without_l = ts.ucmd().arg("-b").arg("dir").succeeds();
+    let size_without_l: u64 = result_without_l
+        .stdout_str()
+        .split('\t')
+        .next()
+        .unwrap()
+        .trim()
+        .parse()
+        .unwrap();
+
+    for arg in ["-l", "--count-links"] {
+        let result_with_l = ts.ucmd().arg("-b").arg(arg).arg("dir").succeeds();
+        let size_with_l: u64 = result_with_l
+            .stdout_str()
+            .split('\t')
+            .next()
+            .unwrap()
+            .trim()
+            .parse()
+            .unwrap();
+
+        assert!(
+            size_with_l >= size_without_l,
+            "With {arg}, size ({size_with_l}) should be >= size without -l ({size_without_l})"
+        );
+    }
+}
+
 #[test]
 fn test_du_h_flag_empty_file() {
     new_ucmd!()
@@ -979,7 +1130,7 @@ fn test_du_threshold() {
     at.write("subdir/links/bigfile.txt", &"x".repeat(10000)); // ~10K file
     at.write("subdir/deeper/deeper_dir/smallfile.txt", "small"); // small file
 
-    let threshold = if cfg!(windows) { "7K" } else { "10K" };
+    let threshold = "10K";
 
     ts.ucmd()
         .arg("--apparent-size")
@@ -996,6 +1147,27 @@ fn test_du_threshold() {
         .stdout_contains("deeper_dir");
 }
 
+#[test]
+#[cfg(not(target_os = "openbsd"))]
+fn test_du_binary_threshold() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+
+    at.mkdir_all("subdir/links");
+    at.mkdir_all("subdir/deeper/deeper_dir");
+    at.write("subdir/links/bigfile.txt", &"x".repeat(10000));
+    at.write("subdir/deeper/deeper_dir/smallfile.txt", "small");
+
+    let threshold_bin = "0b10011100010000";
+
+    ts.ucmd()
+        .arg("--apparent-size")
+        .arg(format!("--threshold={threshold_bin}"))
+        .succeeds()
+        .stdout_contains("links")
+        .stdout_does_not_contain("deeper_dir");
+}
+
 #[test]
 fn test_du_invalid_threshold() {
     let ts = TestScenario::new(util_name!());
@@ -1399,6 +1571,17 @@ fn test_du_files0_from_stdin_with_invalid_zero_length_file_names() {
         .stderr_contains("-:2: invalid zero-length file name");
 }
 
+#[test]
+fn test_du_files0_from_stdin_with_stdin_as_input() {
+    new_ucmd!()
+        .arg("--files0-from=-")
+        .pipe_in("-")
+        .fails_with_code(1)
+        .stderr_is(
+            "du: when reading file names from standard input, no file name of '-' allowed\n",
+        );
+}
+
 #[test]
 fn test_du_files0_from_dir() {
     let ts = TestScenario::new(util_name!());
@@ -1518,7 +1701,7 @@ fn test_du_blocksize_zero_do_not_panic() {
     let ts = TestScenario::new(util_name!());
     let at = &ts.fixtures;
     at.write("foo", "some content");
-    for block_size in ["0", "00", "000", "0x0"] {
+    for block_size in ["0", "00", "000", "0x0", "0b0"] {
         ts.ucmd()
             .arg(format!("-B{block_size}"))
             .arg("foo")
@@ -1820,3 +2003,58 @@ fn test_du_long_path_from_unreadable() {
     perms.set_mode(0o755);
     fs::set_permissions(&inaccessible_path, perms).unwrap();
 }
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_du_hard_links_multiple_dirs_in_args() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+
+    at.mkdir("dir1");
+    at.mkdir("dir2");
+    at.write("dir1/file", "hello world");
+    at.hard_link("dir1/file", "dir2/link");
+
+    let result = ts.ucmd().args(&["dir1", "dir2"]).succeeds();
+    let lines: Vec<&str> = result.stdout_str().lines().collect();
+    let size = |i: usize| lines[i].split_once('\t').unwrap().0.parse::<u64>().unwrap();
+    assert!(size(0) > size(1));
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_du_hard_links_multiple_links_in_args() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+
+    at.mkdir("dir1");
+    at.write("dir1/file", "hello world");
+    at.hard_link("dir1/file", "dir1/link");
+
+    let result = ts.ucmd().args(&["dir1/file", "dir1/link"]).succeeds();
+    result.stdout_contains("dir1/file");
+    result.stdout_does_not_contain("dir1/link");
+
+    let result = ts.ucmd().args(&["-L", "dir1/file", "dir1/link"]).succeeds();
+    result.stdout_contains("dir1/file");
+    result.stdout_does_not_contain("dir1/link");
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_du_symlinks_multiple_links_in_args() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+
+    at.mkdir("dir1");
+    at.write("dir1/file", "hello world");
+    at.symlink_file("dir1/file", "dir1/link");
+
+    let result = ts.ucmd().args(&["dir1/file", "dir1/link"]).succeeds();
+    result.stdout_contains("dir1/file");
+    result.stdout_contains("dir1/link");
+
+    let result = ts.ucmd().args(&["-L", "dir1/file", "dir1/link"]).succeeds();
+    result.stdout_contains("dir1/file");
+    result.stdout_does_not_contain("dir1/link");
+}
